# Copyright (C) 2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

class WindowingTransformations:
	def __init__(self, fs_widget, parent):
		self.parent = parent
		self.fs_widget = fs_widget
		self.hidden_widgets = []
		self.is_fw = False
		self.is_fs = False
	
	def fullscreen(self, fs=True):
		if not self.parent.config_manager.uses_new_bindings():
			if self.is_fw == False:
				self.hidden_widgets = []
				
		self.hide_stuff(self.fs_widget)
		
		if fs == True:
			self.fs_widget.get_toplevel().fullscreen()
			self.is_fs = True
		else:
			self.is_fw = True
		
		self.is_fs = True
		
	def hide_stuff(self, vis_widget):
		parent = vis_widget.get_parent()
		
		if parent is not parent.get_toplevel():
			for w in parent.get_children():
				if w is not vis_widget and w.get_property("visible"):
					self.hidden_widgets.append(w)
					w.hide()
			self.hide_stuff(parent)
		else:
			return
	
	def unfullscreen(self, fs=True):
		if self.is_fs:
			self.fs_widget.get_toplevel().unfullscreen()
			self.is_fs = False
		
		if not fs:
			self.is_fw = False
		
		if (fs and not self.is_fw) or not fs:	
			for w in self.hidden_widgets:
				w.show()
				self.hidden_widgets = []
			
	def fullwindow(self):
		w_width = self.fs_widget.get_toplevel().get_allocation()[2]
		w_height = self.fs_widget.get_toplevel().get_allocation()[3]
		self.prev_width = w_width - self.fs_widget.get_allocation()[2]
		self.prev_height = w_height - self.fs_widget.get_allocation()[3]
		self.fs_widget.get_toplevel().resize(self.fs_widget.get_allocation()[2], self.fs_widget.get_allocation()[3])
		self.fullscreen(fs=False)
		self.is_fw = True
		
	def unfullwindow(self):
		width = self.fs_widget.get_allocation()[2] + self.prev_width
		height = self.fs_widget.get_allocation()[3] + self.prev_height
		self.fs_widget.get_toplevel().resize(width, height)
		self.unfullscreen(fs=False)
		self.is_fw = False
		



# Copyright (C) 2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import locale
import math
import sys
import os
import pySopCastConfigurationManager
import locale
import gettext

cur_locale = locale.setlocale(locale.LC_ALL, "")

gtk.glade.bindtextdomain("sopcast-player")
gtk.glade.textdomain("sopcast-player")

gettext.bindtextdomain("sopcast-player")
gettext.textdomain("sopcast-player")

lang = gettext.translation("sopcast-player", languages=[cur_locale], fallback=True)
lang.install('sopcast-player')

def is_chinese():
	return not locale.setlocale(locale.LC_ALL, "")[:len("zh".lower())] != "zh".lower()

class OptionsDialog:
	def __init__(self, parent):
		self.parent = parent
		self.run()
		
	
	def run(self):	
		gladefile = "%s/%s" % (os.path.realpath(os.path.dirname(sys.argv[0])), "../ui/Options.glade")
		
		tree = gtk.glade.XML(gladefile, "window")
	
		dialog = tree.get_widget("window")
		dialog.set_title("%s - %s" % (_("Preferences"), "SopCast Player"))
		dialog.set_transient_for(self.parent.window)
			
		# Default retrieval
		static_ports_default = self.parent.config_manager.use_static_ports()
		inbound_port_default = self.parent.config_manager.inbound_static_port()
		outbound_port_default = self.parent.config_manager.outbound_static_port()
	
		external_player_default = self.parent.config_manager.use_external_player()
		external_player_command_default = self.parent.config_manager.external_player_command()
	
		channel_guide_url_default = self.parent.config_manager.channel_guide_url()
		language_combobox_default = self.parent.config_manager.channel_guide_language()

		# Widget variables
		static_ports = tree.get_widget("static_ports")
		inbound_port = tree.get_widget("inbound_port")
		outbound_port = tree.get_widget("outbound_port")
		static_ports_children = ( inbound_port, outbound_port )
	
		external_player = tree.get_widget("external_player")
		external_player_command = tree.get_widget("external_player_command")
	
		channel_guide_url = tree.get_widget("channel_guide_url")
	
		language_combobox = tree.get_widget("language_combobox")
	
	
		# Signal functions and helpers
		def set_widgets_sensitive(widgets, sensitive):
			for widget in widgets:
				widget.set_sensitive(sensitive)
	
		def on_static_ports_toggled(src, data=None):
			set_widgets_sensitive(static_ports_children, src.get_active())
			self.parent.config_manager.use_static_ports(src.get_active())
			self.parent.static_ports = src.get_active()
		
			if self.parent.static_ports == False:
				self.parent.inbound_port = None
				self.parent.outbound_port = None
			else:
				self.parent.inbound_port = int(inbound_port.get_value())
				self.parent.outbound_port = int(outbound_port.get_value())
	
		def on_inbound_port_value_changed(src, data=None):
			if src.get_value() == outbound_port.get_value():
				src.set_value(src.get_value() + 1)
			else:
				self.parent.config_manager.inbound_static_port(int(src.get_value()))
				self.parent.inbound_port = int(src.get_value())
		
		def on_outbound_port_value_changed(src, data=None):
			if src.get_value() == inbound_port.get_value():
				src.set_value(src.get_value() + 1)
			else:
				self.parent.config_manager.outbound_static_port(int(src.get_value()))
				self.parent.outbound_port = int(src.get_value())
	
		def on_external_player_toggled(src, data=None):
			external_player_command.set_sensitive(src.get_active())
			#print src.get_active()
			self.parent.config_manager.use_external_player(src.get_active())
		
			if src.get_active():
				self.parent.external_player_command = external_player_command.get_text()
				self.parent.set_media_player_visible(False)
			else:
				self.parent.external_player_command = None
				self.parent.set_media_player_visible(True)
			#TODO: Mashup the player window to only show channel guide and set ui_worker to launch external command
	
		def on_external_player_command_focus_out_event(src, event, data=None):
			if external_player_command.get_text() != external_player_command_default:
				self.parent.config_manager.external_player_command(src.get_text())
			
				self.parent.external_player_command = external_player_command.get_text()
				#TODO: Mashup the player window to only show channel guide and set ui_worker to launch external command
	
		def on_channel_guide_url_focus_out_event(src, event, data=None):
			if src.get_text() != channel_guide_url_default:
				self.parent.config_manager.channel_guide_url(src.get_text())
				self.parent.channel_guide_url = src.get_text()
	
		def on_language_combobox_changed(src, data=None):
			chinese = False
			self.parent.config_manager.channel_guide_language(src.get_active_text())
		
			if src.get_active_text() == _("Chinese"):
				chinese = True
		
			self.parent.populate_channel_treeview(chinese)
			self.parent.channel_guide_language = src.get_active_text()
				
				
		# Setup widget defaults
		static_ports.set_active(static_ports_default)
		set_widgets_sensitive(static_ports_children, static_ports_default)
		inbound_port.set_value(inbound_port_default)
		outbound_port.set_value(outbound_port_default)
	
		external_player.set_active(external_player_default)
		external_player_command.set_text(external_player_command_default)
		external_player_command.set_sensitive(external_player_default)
	
		channel_guide_url.set_text(channel_guide_url_default)
	
		if is_chinese() == False:
			language_combobox.insert_text(0, _("English"))
			language_combobox.insert_text(1, _("Chinese"))
		else:
			language_combobox.insert_text(0, _("Chinese"))				
			language_combobox.insert_text(1, _("English"))
	
		if is_chinese() == False:
			if language_combobox_default == _("English"):
				language_combobox.set_active(0)
			else:
				language_combobox.set_active(1)
		else:
			if language_combobox_default == _("Chinese"):
				language_combobox.set_active(0)
			else:
				language_combobox.set_active(1)

		# Signal connect
		dic = { "on_static_ports_toggled" : on_static_ports_toggled,
			"on_inbound_port_value_changed" : on_inbound_port_value_changed,
			"on_outbound_port_value_changed" : on_outbound_port_value_changed,
			"on_external_player_toggled" : on_external_player_toggled,
			"on_external_player_command_focus_out_event" : on_external_player_command_focus_out_event,
			"on_channel_guide_url_focus_out_event" : on_channel_guide_url_focus_out_event,
			"on_language_combobox_changed" : on_language_combobox_changed}
		tree.signal_autoconnect(dic)
	
		dialog.run()
	
		# Post-response action area
		dialog.destroy()



# ConfigurationManager eases the use of ConfigParser
# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

import ConfigParser

class ConfigurationManager(object):
	def __init__(self, file_name):
		self.__file_name = file_name
		self.__sections = { }
	
	def add_section(self, section_name, section):
		self.__sections[section_name] = section
	
	def read(self):
		config = ConfigParser.RawConfigParser()
		config.read(self.__file_name)
		
		for section_name, attributes in self.__sections.iteritems():
			for attribute_name, attribute_value in attributes.iteritems():
				if config.has_section(section_name) == True:
					if config.has_option(section_name, attribute_name):
						attributes[attribute_name] = config.get(section_name, attribute_name)
		
	def write(self):
		config = ConfigParser.RawConfigParser()
		
		for section_name, attributes in self.__sections.iteritems():
			config.add_section(section_name)
			for attribute_name, attribute_value in attributes.iteritems():
				config.set(section_name, attribute_name, str(attribute_value))
			
		configfile = open(self.__file_name, 'wb')
		config.write(configfile)
		
	def set(self, section, attribute, value):
		#print value
		self.__retrieve_attribute(section, attribute, value)
		
	def get(self, section, attribute):
		return self.__retrieve_attribute(section, attribute)
		
	def getint(self, section, attribute):
		return int(self.get(section, attribute))
	
	def getfloat(self, section, attribute):
		return float(self.get(section, attribute))
		
	def getboolean(self, section, attribute):
		return self.__parse_bool(self.get(section, attribute))
		
	def update_configuration_definitions(self):
		self.write()
		
	def __retrieve_attribute(self, section, attribute, value=None):
		config = ConfigParser.SafeConfigParser()
		config.read(self.__file_name)
		
		r_section_name, r_section = self.__retrieve_section(section)
		
		for attribute_name, attribute_value in r_section.iteritems():
			if attribute_name == attribute:
				ret_val = attribute_value
				break
				
		if value != None:
			if ret_val != None:
				r_section[attribute_name] = str(value)
						
		if ret_val != None:
			return ret_val
		else:
			raise ConfigParser.NoOptionError
			
	def __retrieve_section(self, section):
		found = False
		
		for section_name, section_dict in self.__sections.iteritems():
			if section_name == section:
				found = True
				break
		
		if found == True:
			return section_name, section_dict
		else:
			raise ConfigParser.NoSectionError
			
	def __parse_bool(self, s):
		return not s in [False, 'False', 'false', 'f', 'n', '0', '']




#! /usr/bin/python
# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


# Begining 2011 review of code, started 30 April 2011

import gettext
import gobject
import gtk
import gtk.glade
import locale
import math
import sys
import os

from DatabaseOperations import DatabaseOperations
from dynamic_ports import DynamicPorts
from fork import ForkSOP
from listen import SOPStats
import pySocket
from pySopCastConfigurationManager import pySopCastConfigurationManager
from VLCWidget import VLCWidget
from OptionsDialog import OptionsDialog
from WindowingTransformations import WindowingTransformations
from SopcastPlayerWorkerThread import UpdateUIThread
from ChannelGuideWorkerThread import UpdateChannelGuideThread
from OpenSopAddress import OpenSopAddress

cur_locale = locale.setlocale(locale.LC_ALL, "")

gtk.glade.bindtextdomain("sopcast-player")
gtk.glade.textdomain("sopcast-player")

gettext.bindtextdomain("sopcast-player")
gettext.textdomain("sopcast-player")

lang = gettext.translation("sopcast-player", languages=[cur_locale], fallback=True)
lang.install('sopcast-player')

def is_chinese():
	return not cur_locale[:len("zh".lower())] != "zh".lower()

class pySopCast(object):
	def __init__(self, channel_url=None, inbound_port=None, outbound_port=None):
		gtk.gdk.threads_init()
		#*****************Sopcast specific code*******************
		self.vlc= None
		self.ui_worker = None
		self.channel_url = channel_url
		self.static_ports = False
		self.inbound_port = inbound_port
		self.outbound_port = outbound_port
		self.sop_stats = None
		#used by external player
		self.outbound_media_url = None
		self.channel_guide_language = None
		self.sopcast_client_installed = True
		self.channel_guide_worker = None
		self.channel_guide_url = None		
		self.external_player_command = None
		self.fullwindow = False
		try:
			self.fork_sop = ForkSOP()
		except:
			self.sopcast_client_installed = False				
		#************************End******************************
		
		self.status_bar_text = None
		self.db_operations = DatabaseOperations()		
		self.treeview_selection = None
		self.window_title = "SopCast Player"
		self.config_manager = None
		
				
	def main(self, sop_address=None, sop_address_name=None):
		gladefile = "%s/%s" % (os.path.realpath(os.path.dirname(sys.argv[0])), "../ui/pySopCast.glade")
			
		self.glade_window = gtk.glade.XML(gladefile, "window", "sopcast-player")
		self.window = self.glade_window.get_widget("window")
		
		glade_context_menu = gtk.glade.XML(gladefile, "context_menu", "sopcast-player")
		self.context_menu = glade_context_menu.get_widget("context_menu")
		
		glade_bookmarks_context_menu = gtk.glade.XML(gladefile, "bookmarks_context_menu", "sopcast-player")
		self.bookmarks_context_menu = glade_bookmarks_context_menu.get_widget("bookmarks_context_menu")
		

		
		window_signals = { "on_mainWindow_destroy" : self.on_exit,
			"on_play_button_clicked" : self.on_play_button_clicked,
			"on_menu_quit_activate" : self.on_menu_quit_activate,
			"on_menu_fullscreen_activate" : self.on_fullscreen_activate,
			"on_menu_add_bookmark_activate" : self.on_add_bookmark,
			"on_stop_clicked" : self.on_stop_clicked,
			"on_volume_adjust_bounds" : self.on_volume_adjust_bounds,
			"on_menu_about_activate" : self.on_menu_about_activate,
			"on_open_sop_address_activate" : self.on_open_sop_address_activate,
			"on_refresh_channel_guide_clicked" : self.on_refresh_channel_guide_clicked,
			"on_menu_screenshot_activate" : self.on_menu_screenshot_activate,
			"on_menu_preferences_activate" : self.on_menu_preferences_activate,
			"on_menu_stay_on_top_toggled" : self.on_menu_stay_on_top_toggled,
			"on_menu_show_controls_toggled" : self.on_menu_show_controls_toggled,
			"on_window_key_press_event" : self.on_window_key_press_event,
			"on_channel_treeview_button_press_event" : self.on_channel_treeview_button_press_event }
			
		self.glade_window.signal_autoconnect(window_signals)
		
		context_menu_signals = { "on_context_menu_play_activate" : self.on_context_menu_play_activate,
					 "on_context_menu_properties_activate" : self.on_context_menu_properties_activate }
		glade_context_menu.signal_autoconnect(context_menu_signals)
		
		bookmarks_context_menu_signals = { "on_bookmarks_context_delete_activate" : self.on_bookmarks_context_delete_activate, }
		glade_bookmarks_context_menu.signal_autoconnect(bookmarks_context_menu_signals)
		
		#*****************Sopcast specific code*******************
		self.config_manager = pySopCastConfigurationManager()
		self.vlc = VLCWidget(self.eb, self)
		self.player_volume = self.config_manager.player_volume()
		self.volume.set_value(self.player_volume)
		self.ui_worker = UpdateUIThread(self, self.config_manager.channel_timeout())
		self.ui_worker.start()			
		self.channel_guide_url = self.config_manager.channel_guide_url()
		self.channel_guide_hpane.set_position(self.config_manager.channel_guide_pane_position())		
		self.channel_guide_language = self.config_manager.channel_guide_language()		
		
		if self.config_manager.use_external_player() == True:
			self.set_media_player_visible(False)
			self.external_player_command = self.config_manager.external_player_command()
			show_channel_guide_pane = True	
		
		#these four lines don't belong with sc code but must be before populate_channel_treeview
		self.channel_treeview_model = gtk.TreeStore(int, str, str, str, str, str, int, int, int, str)
		self.treeview_selection = self.channel_treeview.get_selection()
		self.treeview_selection_changed_handler = self.treeview_selection.connect("changed", self.on_selection_changed)
		self.channel_treeview.connect("row_activated", self.on_channel_treeview_row_activated)			
				
		chinese = self.channel_guide_language == _("Chinese")
		self.populate_channel_treeview(chinese)		
		
		if sop_address != None:
			if sop_address[:len("sop://".lower())] == "sop://".lower():
				self.play_channel(sop_address, sop_address_name)
		#************************End******************************		
		
		self.window.set_default_size(self.config_manager.player_width(), self.config_manager.player_height())
		self.display_pane.set_position(self.config_manager.display_pane_position())
		show_channel_guide_pane = self.config_manager.show_channel_guide()		
		self.window.set_keep_above(self.config_manager.stay_on_top())		
		self.menu_stay_on_top.set_active(self.config_manager.stay_on_top())
		self.populate_bookmarks()

		column = gtk.TreeViewColumn("Name", gtk.CellRendererText(), text=1)
		self.channel_treeview.append_column(column)
		
		if show_channel_guide_pane == False:
			self.channel_guide_pane.hide()
		
		if not self.config_manager.uses_new_bindings():
			self.eb.add_events(gtk.gdk.BUTTON_PRESS_MASK)
			self.eb.connect("button_press_event", self.on_mouse_button_clicked)
		
		self.wt = WindowingTransformations(self.eb, self)
		self.show_channel_guide.set_active(show_channel_guide_pane)
		self.show_channel_guide.connect("toggled", self.on_show_channel_guide_toggled)
	
		if self.channel_url != None:
			self.play_channel()
		
		self.window.show()
			
		gtk.main()
		
		if self.fork_sop != None:
			self.fork_sop.kill_sop()
			
	def on_mouse_button_clicked(self, widget, event):
		if event.type == gtk.gdk._2BUTTON_PRESS:
			if self.is_playing():
				self.toggle_fullscreen()
		else:
			return True
			
	def toggle_fullscreen(self):
		if not self.fullscreen:
			if self.vlc.media_loaded():
				self.vlc.fullscreen()
				#self.vlc.display_text("         %s" % "Press Esc to exit fullscreen")
				self.fullscreen = not self.fullscreen
		else:
			self.vlc.unfullscreen()
			self.fullscreen = not self.fullscreen
	
	#code for context menu
	def on_context_menu_play_activate(self, src, data=None):
		self.play_channel(self.channel_treeview_model[self.channel_treeview.get_cursor()[0]][9])
	
	def on_context_menu_properties_activate(self, src, data=None):
		path, column = self.channel_treeview.get_cursor()
		
		gladefile = "%s/%s" % (os.path.realpath(os.path.dirname(sys.argv[0])), "../ui/ChannelProperties.glade")
			
		tree = gtk.glade.XML(gladefile, "window")
		
		dialog = tree.get_widget("window")
		dialog.set_title("%s - %s" % (_("Properties"), "SopCast Player"))
		dialog.set_transient_for(self.window)
		
		label_name = tree.get_widget("label_name")
		label_classification = tree.get_widget("label_classification")
		label_stream_type = tree.get_widget("label_stream_type")
		label_bitrate = tree.get_widget("label_bitrate")
		label_qc = tree.get_widget("label_qc")
		label_qs = tree.get_widget("label_qs")
		label_description = tree.get_widget("label_description")
		
		label_group = [label_name, label_classification, label_stream_type, label_bitrate, label_qc, label_qs, label_description]
		labels = ["%s: %s" % (_("Name"), self.html_escape(self.channel_treeview_model[path][1])), "%s: %s" % (_("Classification"), self.html_escape(self.channel_treeview_model[path][4])), "%s: %s" % (_("Stream Format"), self.html_escape(self.channel_treeview_model[path][5].upper())), "%s %d kb/s" % (_("Bitrate:"), self.channel_treeview_model[path][6]), "%s: %d" % (_("QC"), self.channel_treeview_model[path][7]), "%s: %d" % (_("QS"), self.channel_treeview_model[path][8]), "%s: %s" % (_("Description"), self.html_escape(self.channel_treeview_model[path][2]))]
		self.set_label_group(label_group, labels)
		
		dialog.run()
		
		dialog.destroy()
	#end
	
	def on_fullscreen_activate(self, src, data=None):
		if self.config_manager.uses_new_bindings():
			self.vlc.toggle_fullscreen()
		else:
			self.toggle_fullscreen()
		
	def __getattribute__(self, key):
		value = None
		try:
			value = object.__getattribute__(self, key)
		except AttributeError:
			glade_window = object.__getattribute__(self, 'glade_window')
			value = glade_window.get_widget(key)	

		return value
	
	def get_ports(self):
		inbound_port = None
		outbound_port = None
		exit = False
		
		if self.config_manager.use_static_ports() == True:
			inbound_port = self.config_manager.inbound_static_port()
			outbound_port = self.config_manager.outbound_static_port()
			self.static_ports = True
			s = pySocket.pySocket()
			if not s.is_available(self.config_manager.server(), inbound_port) or not s.is_available(self.config_manager.server(), outbound_port):

				dialog = gtk.Dialog(_("Static Ports Unavailable"),
					self.window,
					gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
					(gtk.STOCK_NO, gtk.RESPONSE_REJECT,
					gtk.STOCK_YES, gtk.RESPONSE_ACCEPT))
			
				hbox = gtk.HBox()
		
			
				label = gtk.Label(_("Static ports unavailable, do you wish to continue using dynamic ports?"))
				hbox.pack_start(label)
				hbox.set_size_request(-1, 50)
				hbox.show_all()
				dialog.set_default_response(gtk.RESPONSE_ACCEPT)
				dialog.vbox.pack_start(hbox)
		
				if dialog.run() == gtk.RESPONSE_REJECT:
					exit = True
				
				dialog.destroy()

				dyn = DynamicPorts()
				inbound_port, outbound_port = dyn.get_ports()
				self.static_ports = False
		else:
			dyn = DynamicPorts()
			inbound_port, outbound_port = dyn.get_ports()
		
		if exit == True:
			inbound_port = None
			outbound_port = None
			
		return inbound_port, outbound_port
	
	
	###################################
	def on_bookmarks_button_pressed(self, src, event, bookmark):
		if event.button == 3:
			self.selected_bookmark = bookmark
			self.bookmarks_context_menu.popup(None, None, None, event.button, event.time)
		
	def on_bookmarks_context_delete_activate(self, src, data=None):
		self.db_operations.delete_bookmark(self.selected_bookmark[0])
		self.bookmarks_menu.popdown()
		self.main_menu.cancel()
		self.populate_bookmarks()
		
	def populate_bookmarks(self):
		self.clear_bookmarks()
		for bookmark in self.db_operations.retrieve_bookmarks():
			menu_item = gtk.MenuItem(bookmark[1])
			menu_item.connect("activate", self.on_menu_bookmark_channel_activate, bookmark)
			menu_item.connect("button_press_event", self.on_bookmarks_button_pressed, bookmark)
			self.menu_bookmarks.get_submenu().append(menu_item)
			self.menu_bookmarks.get_submenu().show_all()
	
	def populate_channel_treeview(self, chinese=False):
		self.channel_treeview.set_model()
		self.channel_treeview_model.clear()
		if chinese == False:
			channel_groups = self.db_operations.retrieve_channel_groups()
		else:
			channel_groups = self.db_operations.retrieve_channel_groups_cn()
		
		for channel_group in channel_groups:
			channel_group_iter = self.channel_treeview_model.append(None, self.prepare_row_for_channel_treeview_model(channel_group))
			
			if chinese == False:
				channels = self.db_operations.retrieve_channels_by_channel_group_id(channel_group[0])
			else:
				channels = self.db_operations.retrieve_channels_by_channel_group_id_cn(channel_group[0])
		
			for channel in channels:
				self.channel_treeview_model.append(channel_group_iter, self.prepare_row_for_channel_treeview_model(channel))
		
		self.channel_treeview.set_model(self.channel_treeview_model)

	def set_media_player_visible(self, visible):
		if visible == True:
			if self.show_channel_guide.get_active() == False:
				self.channel_guide_pane.hide()
					
			self.menu_view.show()
			self.media_box.show()
			self.channel_properties_pane.hide()
		else:
			self.menu_view.hide()
			self.media_box.hide()
			self.channel_properties_pane.show()
			
			if self.channel_guide_pane.get_property("visible") == False:
				self.channel_guide_pane.show()
	
	def prepare_row_for_channel_treeview_model(self, row):
		if len(row) == 10:
			return row
		else:
			return [row[0], row[1], row[2], None, None, None, 0, 0, 0, None]
			
	def clear_bookmarks(self):
		menu_items = self.menu_bookmarks.get_submenu().get_children()
		
		if len(menu_items) > 2:
			i = 2
			while i < len(menu_items):
				self.menu_bookmarks.get_submenu().remove(menu_items[i])
				i += 1
	
	def on_menu_quit_activate(self, src, data=None):
		self.window.destroy()
		
	def on_menu_bookmark_channel_activate(self, src, channel_info):
		self.play_channel(channel_info[2])
		
	def on_add_bookmark(self, src, data=None):
		dialog = gtk.Dialog(_("Channel Bookmark"),
			self.window,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
			gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
			
		hbox = gtk.HBox()
			
		label = gtk.Label("%s %s" % (_("Name"), ": "))
		entry = gtk.Entry()
		
		entry.connect("activate", lambda a: dialog.response(gtk.RESPONSE_ACCEPT))
		
		if self.selection != None:
			if len(entry.get_text()) > 0:
				entry.set_text(self.selection[1])
		
		if self.selection:
			entry.set_text(self.selection[1])
		
		hbox.pack_start(label)
		hbox.pack_start(entry)
		hbox.set_size_request(-1, 50)
		hbox.show_all()
		dialog.set_default_response(gtk.RESPONSE_ACCEPT)
		dialog.vbox.pack_start(hbox)
		
		if dialog.run() == gtk.RESPONSE_ACCEPT:
			if len(entry.get_text()) > 0:
				self.add_bookmark(unicode(entry.get_text()), self.url)
			
		dialog.destroy()
		
	def on_stop_clicked(self, src, data=None):
		self.menu_fullscreen.set_sensitive(False)
		self.vlc.stop_media()
		
		self.ui_worker.shutdown()
		if self.fork_sop.is_running() == True:
			self.fork_sop.kill_sop()
		
		self.update_statusbar("")
		self.window.set_title(self.window_title)
		
	def on_volume_adjust_bounds(self, src, data=None):
		self.player_volume = int(src.get_value())
		self.vlc.set_volume(self.player_volume)
	
	def on_menu_about_activate(self, src, data=None):
		gladefile = "%s/%s" % (os.path.realpath(os.path.dirname(sys.argv[0])), "../ui/About.glade")
		about_file = gtk.glade.XML(gladefile, "about")
		about = about_file.get_widget("about")
		about.set_transient_for(self.window)
		about.run()
		about.destroy()
	
	def on_open_sop_address_activate(self, src, data=None):
		dialog = gtk.Dialog(_("Open Sop Address"),
			self.window,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
			gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
			
		hbox = gtk.HBox()
			
		label = gtk.Label("%s %s" % (_("Enter Sop Address"), ": "))
		entry = gtk.Entry()
		
		clipboard = gtk.clipboard_get().wait_for_text()
		
		if clipboard[0:6].lower() == "sop://":
			entry.set_text(clipboard)
		
		############# Code contribution by Benjamin Kluglein ####################
		entry.connect("activate", lambda a: dialog.response(gtk.RESPONSE_ACCEPT))
		#########################################################################
		
		hbox.pack_start(label)
		hbox.pack_start(entry)
		hbox.set_size_request(-1, 50)
		hbox.show_all()
		dialog.set_default_response(gtk.RESPONSE_ACCEPT)
		dialog.vbox.pack_start(hbox)
		
		if dialog.run() == gtk.RESPONSE_ACCEPT:
			if len(entry.get_text()) > 0:
				self.play_channel(entry.get_text())
			
		dialog.destroy()
	
	def on_show_channel_guide_toggled(self, src, data=None):
		window_width = self.window.get_size()[0]
		window_height = self.window.get_size()[1]
		handle_width = self.display_pane.style_get_property("handle-size")
		
		if src.get_active() == True:
			self.window.resize(window_width + self.config_manager.channel_guide_width(), window_height)
			self.channel_guide_pane.show()
			
		else:
			pane2_width = self.display_pane.get_child2().get_allocation()[2]
			self.config_manager.channel_guide_width(pane2_width + handle_width)
			
			self.channel_guide_pane.hide()
			self.window.resize(self.display_pane.get_child1().get_allocation()[2], window_height)
	
	def on_channel_treeview_row_activated(self, treeview, path, view_column, data=None):
		if self.channel_treeview_model.iter_has_child(treeview.get_model().get_iter(path)) == True:
			if self.channel_treeview.row_expanded(path) == False:
				self.channel_treeview.expand_row(path, False)
			else:
				self.channel_treeview.collapse_row(path)
		else:
			self.play_channel(self.selection[9], self.selection[1])
			
	def set_label_group(self, label_group, labels=None):
		i = 0
		if labels != None:
			while i < len(label_group):
				label_group[i].set_label(labels[i])
				i += 1
		else:
			while i < len(label_group):
				label_group[i].set_label("")
				i += 1

	def on_selection_changed(self, src, data=None):
		model, s_iter = src.get_selected()

		if s_iter:
			row = model.get_path(s_iter)
			self.selection = self.channel_treeview_model[row]
	
			if self.channel_treeview_model.iter_has_child(s_iter) == False:
				label_group = [self.label_name, self.label_channel_group, self.label_classification, self.label_stream_type, self.label_bitrate, self.label_qc, self.label_qs, self.label_description]
				labels = ["%s: %s" % (_("Name"), self.html_escape(self.selection[1])), "%s: %s" % (_("Channel Group"), self.html_escape(self.channel_treeview_model[self.channel_treeview_model.get_path(self.channel_treeview_model.iter_parent(s_iter))][1])), "%s: %s" % (_("Classification"), self.html_escape(self.selection[4])), "%s: %s" % (_("Stream Format"), self.html_escape(self.selection[5].upper())), "Bitrate: %d kb/s" % self.selection[6], "%s: %d" % (_("QC"), self.selection[7]), "%s: %d" % (_("QS"), self.selection[8]), "%s: %s" % (_("Description"), self.html_escape(self.selection[2]))]
				self.set_label_group(label_group, labels)
		
			else:
				label_group = [self.label_name, self.label_channel_group, self.label_classification, self.label_stream_type, self.label_bitrate, self.label_qc, self.label_qs, self.label_description]
				labels = ["%s: %s" % (_("Name"), self.html_escape(self.selection[1])), "%s: %d" % (_("Channels"), self.get_iter_child_count(s_iter)), "%s: %s" % (_("Description"), self.html_escape(self.selection[2])), "" ,"" ,"" ,"" ,""]
				self.set_label_group(label_group, labels)
		else:
			self.selection = None
			label_group = [self.label_name, self.label_channel_group, self.label_classification, self.label_stream_type, self.label_bitrate, self.label_qc, self.label_qs, self.label_description]
			self.set_label_group(label_group)
	
	def on_refresh_channel_guide_clicked(self, src, data=None):
		self.update_channel_guide_progress.set_fraction(0)
		self.channel_guide_label.hide()
		self.update_channel_guide_progress.show()
	
		if self.channel_guide_worker != None:
			if self.channel_guide_worker.running == False:
				self.channel_guide_worker = None
			
				self.channel_guide_worker = UpdateChannelGuideThread(self)
				self.channel_guide_worker.start()
		else:
			self.channel_guide_worker = UpdateChannelGuideThread(self)
			self.channel_guide_worker.start()
	
	def on_menu_screenshot_activate(self, src, data=None):
		screenshot = self.vlc.screenshot()
		print(screenshot)
	
	def on_menu_preferences_activate(self, src, data=None):
		od = OptionsDialog(self)
	
	def on_menu_stay_on_top_toggled(self, src, data=None):
		self.window.set_keep_above(src.get_active())
	
	def on_menu_show_controls_toggled(self, src, data=None):
		self.toggle_menu_controls()
		
	def toggle_menu_controls(self):
		if self.config_manager.uses_new_bindings():
			self.vlc.toggle_fullwindow()
		else:
			if self.fullwindow:
				self.wt.unfullwindow()
			else:
				self.wt.fullwindow()
		
			self.fullwindow = not self.fullwindow

	def on_window_key_press_event(self, src, event, data=None):
		if not self.config_manager.uses_new_bindings():
			if gtk.gdk.keyval_name(event.keyval) in ["h", "H"]:
				if not self.fullscreen:
					self.toggle_menu_controls()
				return True
			elif event.keyval == gtk.keysyms.Escape:
				if self.fullscreen:
					self.toggle_fullscreen()
				return True
			elif gtk.gdk.keyval_name(event.keyval) in ["f", "F"]:
				self.menu_fullscreen.activate()
				return True
		
		if gtk.gdk.keyval_name(event.keyval) in ["t", "T"]:
			active = not self.menu_stay_on_top.get_active()
			self.menu_stay_on_top.set_active(active)
			return True
		elif gtk.gdk.keyval_name(event.keyval) in ["q", "Q"]:
			self.menu_quit.activate()
			return True
		elif gtk.gdk.keyval_name(event.keyval) in ["o", "O"]:
			self.open_sop_address.activate()
			return True
		elif gtk.gdk.keyval_name(event.keyval) in ["d", "D"]:
			if self.menu_add_bookmark.get_sensitive():
				self.menu_add_bookmark.activate()
				return True
		
		return False
	
	def on_channel_treeview_button_press_event(self, src, event, data=None):
		if event.button == 3:
			x = int(event.x)
			y = int(event.y)
			time = event.time
			pthinfo = self.channel_treeview.get_path_at_pos(x, y)
			if pthinfo is not None:
				path, col, cellx, celly = pthinfo
				self.channel_treeview.grab_focus()
				self.channel_treeview.set_cursor( path, col, 0 )
				if self.channel_treeview_model.iter_has_child(self.channel_treeview_model.get_iter(path)) == False:
					self.context_menu.popup( None, None, None, event.button, time)
		return False

	def get_iter_child_count(self, parent_iter):
		i = 0
		
		child = self.channel_treeview_model.iter_children(parent_iter)
		
		while child != None:
			child = self.channel_treeview_model.iter_next(child)
			i += 1
		
		return i
		
	def update_status_bar_text(self, txt):
		if self.status_bar != None:
			self.status_bar_text = txt
	
	def set_volume(self, volume):
		self.vlc.set_volume(volume)
		
	def start_vlc(self):
		if self.config_manager.uses_new_bindings():
			self.vlc.play_media()
			self.vlc.set_volume(self.player_volume)
			return True
		else:
			if self.vlc.get_parent() == None:
				self.eb.add(self.vlc)
				self.eb.show_all()
				return False
		
			if self.vlc.get_parent() == self.eb:
				self.vlc.play_media()
				self.vlc.set_volume(self.player_volume)
				return True
			else:
				return False
			
	def stop_vlc(self):
		self.vlc.stop_media()
		
	def play_channel(self, channel_url=None, title=None):
		self.menu_add_bookmark.set_sensitive(True)
		if self.fork_sop != None:
			if self.fork_sop.is_running() == True:
				self.fork_sop.kill_sop()
		
			s = pySocket.pySocket()
			if (self.inbound_port == None or self.outbound_port == None) or (not s.is_available(self.config_manager.server(), self.inbound_port) or not s.is_available(self.config_manager.server(), self.outbound_port)):
				self.inbound_port, self.outbound_port = self.get_ports()
			
			if self.inbound_port == None or self.outbound_port == None:
				self.window.destroy()
			else:
				self.sop_stats = SOPStats(self.config_manager.server(), self.outbound_port)
			
				if channel_url != None:
					self.channel_url = channel_url
					if title == None:
						self.window.set_title("%s - %s" % (channel_url, self.window_title))
			
				if title != None:
					self.window.set_title("%s - %s" % (title, self.window_title))
				else:
					records = self.db_operations.retrieve_bookmark_by_address(self.channel_url)
					if len(records) > 0:
						self.window.set_title("%s - %s" % (records[0][0], self.window_title))
					else:
						records = self.db_operations.retrieve_channel_guide_record_by_address(self.channel_url)
						if len(records) > 0:
							self.window.set_title("%s - %s" % (records[0][0], self.window_title))
			
				self.fork_sop.fork_sop(self.channel_url, str(self.inbound_port), str(self.outbound_port))
		
				self.menu_add_bookmark.set_sensitive(True)
				self.menu_fullscreen.set_sensitive(True)
		
				self.outbound_media_url = "http://%s:%d/tv.asf" % (self.config_manager.server(), self.outbound_port)
				self.vlc.set_media_url(self.outbound_media_url)
				self.ui_worker.startup()

	def on_play_button_clicked(self, src, data=None):
		if self.channel_url != None:
			self.play_channel(self.channel_url)
		
	def lookup_title(self, title):
		if self.title[:len("sop://")] == "sop://".lower():
			records = self.db_operations.retrieve_bookmark_by_address(channel_url)
			if len(records) > 0:
				self.channel_url_entry.set_text(records[0][0])
			else:
				records = self.db_operations.retrieve_channel_guide_record_by_address(channel_url)
				if len(records) > 0:
					self.channel_url_entry.set_text(records[0][0])
				else:
					self.play_channel(channel_url)
		else:
			
			if len(records) > 0:
				self.play_channel(records[0][2], title)
			
	def on_exit(self, widget, data=None):
		self.config_manager.player_width(self.window.get_allocation()[2])
		self.config_manager.player_height(self.window.get_allocation()[3])
		
		if self.channel_selection_pane.get_property("visible") == True and self.media_box.get_property("visible") == True:	
			self.config_manager.display_pane_position(self.display_pane.get_position())
		
		if self.channel_properties_pane.get_property("visible") == True and self.media_box.get_property("visible") == False:
			self.config_manager.channel_guide_pane_position(self.channel_guide_hpane.get_position())
			
		self.config_manager.show_channel_guide(self.show_channel_guide.get_active())
		self.config_manager.player_volume(int(self.volume.get_value()))
		
		self.ui_worker.stop()
		if self.fork_sop != None:
			if self.fork_sop.is_running() == True:
				self.vlc.stop_media()
				self.vlc.exit_media()
				self.fork_sop.kill_sop()
			
		gtk.main_quit()
		self = None
	
	def add_bookmark(self, channel_name, url=None):
		if url == None:
			url = self.channel_url
		
		self.db_operations.insert_bookmark(channel_name, url)
		self.populate_bookmarks()
		
		if url == self.channel_url:
			self.window.set_title("%s - %s" % (channel_name, self.window_title))
	
	def update_statusbar(self, text):
		if self.status_bar != None:
			self.status_bar.push(1, text)
		
	def set_title(self, title="pySopCast"):
		self.window.set_title(title)	
		
	def html_escape(self, text):
		html_escape_table = {
			"&": "&amp;",
			'"': "&quot;",
			"'": "&apos;",
			">": "&gt;",
			"<": "&lt;",
			}
		"""Produce entities within text."""
		L=[]
		for c in text:
			L.append(html_escape_table.get(c,c))
		return "".join(L)
		
if __name__ == '__main__':
	def print_usage_and_exit():
		print("Usage: sopcast-player [SOP_ADDRESS] [IN-BOUND_PORT OUT-BOUND_PORT]")
		sys.exit(1)
		
	if len(sys.argv) > 1:
		if len(sys.argv) == 2 and sys.argv[1][:len("sop://".lower())] == "sop://".lower():
				pySop = pySopCast(sys.argv[1])
				pySop.main()
				
		elif len(sys.argv) == 4 and sys.argv[1][:len("sop://".lower())] == "sop://".lower():
			try:
				pySop = pySopCast(sys.argv[1], int(sys.argv[2]), int(sys.argv[3]))
				pySop.main()
			except ValueError:
				print_usage_and_exit()
		else:
			print_usage_and_exit()
	else:
		pySop = pySopCast()
		pySop.main()
	



# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sqlite3

class DatabaseOperations(object):
	def __init__(self):
		create_tables = False
		if os.path.isdir(os.path.expanduser(r'~/.pySopCast/')) == False:
			os.mkdir(os.path.expanduser('~/.pySopCast/'))
			
		if os.path.exists(os.path.expanduser(r'~/.pySopCast/pySopCast.db')) == False:
			create_tables = True
			

		if create_tables == True:
			conn = self.db_connect()
			cursor = conn.cursor()		
			cursor.execute('CREATE TABLE bookmarks (id INTEGER PRIMARY KEY, channel_name VARCHAR(100), channel_url VARCHAR(255))')
			cursor.execute('CREATE TABLE channel_groups (id INTEGER PRIMARY KEY, type INT, en_name VARCHAR(255), cn_name VARCHAR(255), ' \
				'name VARCHAR(255), description VARCHAR(1024))')
			cursor.execute('CREATE TABLE channels (id INTEGER PRIMARY KEY, type INTEGER, btype INTEGER, language VARCHAR(255), ' \
				'en_name VARCHAR(255), cn_name VARCHAR(255), name VARCHAR(255), status INTEGER, region_en VARCHAR(255), ' \
				'region_cn VARCHAR(255), region VARCHAR(255), class_en VARCHAR(255), class_cn VARCHAR(255), class INTEGER, ' \
				'user_count INTEGER, sn INTEGER, visit_count INTEGER, start_from VARCHAR(255), stream_type VARCHAR(255), ' \
				'kbps INTEGER, qs INTEGER, qc INTEGER, sop_address VARCHAR(255), cn_description VARCHAR(1024), ' \
				'description VARCHAR(1024), channel_group_id INTEGER)')
			
			conn.commit()	
			conn.close()
			
		self.conn = None
		self.cursor = None
			
	def db_connect(self):			
		return sqlite3.connect(os.path.expanduser('~/.pySopCast/pySopCast.db'))
			
	def insert_bookmark(self, channel_name, channel_url):
		conn = self.db_connect()
		row = (channel_name, channel_url)
		cursor = conn.cursor()
		cursor.execute('INSERT INTO bookmarks VALUES (null, ?, ?)', row)
		conn.commit()
		conn.close()
		
	def insert_channel_group(self, row):
		if not self.conn:
			self.conn = self.db_connect()
			self.cursor = self.conn.cursor()
			
		self.cursor.execute('INSERT INTO channel_groups VALUES (?, ?, ?, ?, ?, ?)', row)
	
	def commit_channel_guide(self):
		self.cursor.execute('VACUUM')
		self.conn.commit()
		self.conn.close()
		self.conn = None
		self.cursor = None

		
	def insert_channel(self, row):
		if not self.conn:
			self.conn = self.db_connect()
			self.cursor = self.conn.cursor()
		
		self.cursor.execute('INSERT INTO channels VALUES (null, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', row)
		
	def truncate_channels(self):
		if not self.conn:
			self.conn = self.db_connect()
			self.cursor = self.conn.cursor()
			
		self.cursor.execute('DELETE FROM channels')
		
	def retrieve_other_channel_group_id(self):
		conn = self.db_connect()
		cursor = conn.cursor()
		cursor.execute("SELECT id FROM channel_groups WHERE name='Other'")
		data = cursor.fetchall()
		conn.close()
		return data
		
	def truncate_channel_groups(self):
		if not self.conn:
			self.conn = self.db_connect()
			self.cursor = self.conn.cursor()
		self.cursor.execute('DELETE FROM channel_groups')
		
	def retrieve_bookmark_by_channel_name(self, channel_name):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = [channel_name.lower()]
		cursor.execute("SELECT * FROM bookmarks WHERE lower(channel_name)=?", row)
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_bookmark_by_address(self, sop_address):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = [sop_address.lower()]
		cursor.execute("SELECT channel_name FROM bookmarks WHERE lower(channel_url)=?", row)
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_channel_guide_record_by_address(self, sop_address):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = [sop_address.lower()]
		cursor.execute("SELECT en_name, cn_name, name FROM channels WHERE lower(sop_address)=?", row)
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_channel_guide_record_by_channel_name(self, channel_name):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = [channel_name.lower()]
		cursor.execute("SELECT sop_address FROM channels WHERE lower(en_name)=?", row)
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_channels_by_channel_group_id(self, channel_group_id):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = [channel_group_id]
		cursor.execute("SELECT id, en_name, description, region, class_en, stream_type, kbps, qs, qc, sop_address FROM channels WHERE channel_group_id=?", row)
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_channels_by_channel_group_id_cn(self, channel_group_id):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = [channel_group_id]
		cursor.execute("SELECT id, cn_name, cn_description, region_cn, class_cn, stream_type, kbps, qs, qc, sop_address FROM channels WHERE channel_group_id=?", row)
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_channel_groups(self):
		conn = self.db_connect()
		cursor = conn.cursor()
		cursor.execute("SELECT id, en_name, description FROM channel_groups")
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_channel_groups_cn(self):
		conn = self.db_connect()
		cursor = conn.cursor()
		cursor.execute("SELECT id, cn_name, description FROM channel_groups")
		data = cursor.fetchall()
		conn.close()
		return data
		
	def retrieve_bookmarks(self):
		conn = self.db_connect()
		cursor = conn.cursor()
		cursor.execute('SELECT * FROM bookmarks')
		data = cursor.fetchall()
		conn.close()
		return data
		
	def delete_bookmark(self, row_id):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = (row_id)
		cursor.execute("DELETE FROM bookmarks WHERE id=%s" % row_id)
		conn.commit()
		conn.close()
		
	def update_bookmark(self, bookmark_name, bookmark_url, bookmark_id):
		conn = self.db_connect()
		cursor = conn.cursor()
		row = (bookmark_name, bookmark_url, bookmark_id)
		cursor.execute('UPDATE bookmarks SET bookmark_name=?, bookmark_url=? WHERE id=?', row)
		conn.commit()
		conn.close()



# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os

class UserPathCheck:
	def __init__(self, file_name):
		self.file_name = file_name
	
	def __get_user_path_array(self):
		return os.environ['PATH'].split(':')
	
	def file_exists(self):
		file_exists = False
		for path in self.__get_user_path_array():
			if os.path.isfile(os.path.join(path, self.file_name)):
				file_exists = True
				break
		return file_exists
			



# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import signal
import sys
import time
import pyUserPathCheck

class ForkSOP:
	def __init__(self, sop_address=None, inbound_port=None, outbound_port=None):
		self.child_pid = None
		self.sop_address = sop_address
		self.inbound_port = inbound_port
		self.outbound_port = outbound_port
		if not pyUserPathCheck.UserPathCheck('sp-sc').file_exists():
			raise
		
	def fork_sop(self, sop_address=None, inbound_port=None, outbound_port=None):
		self.sop_address = sop_address
		self.inbound_port = inbound_port
		self.outbound_port = outbound_port
		
		if self.sop_address == None or self.inbound_port == None or self.outbound_port == None:
			perror("invalid call to fork_sop")
		else:
			pid = os.fork();
			if pid == -1: #fork error
				perror("fork")
				self.child_pid = None
			elif pid == 0: #execute in child
				stdout_file = sys.stdout.fileno()
				sys.stdout.close()
				os.close(stdout_file)
				sys
				os.execlp("sp-sc", "sp-sc", self.sop_address, self.inbound_port, self.outbound_port)
			else: #child's pid, main process execution
				self.child_pid = pid
			
	def kill_sop(self):
		if self.is_running() == True:
			try:
				os.kill(self.child_pid, signal.SIGKILL)
				killedpid, stat = os.wait()
			except OSError:
				sys.stderr.write("Process %s does not exist\n" % self.child_pid)
	
	def is_running(self):
		if self.child_pid != None:
			try:
				os.kill(self.child_pid, 0)
				return True
			except OSError:
				return False
		else:
			return False

class ForkExternalPlayer:
	def __init__(self):
		self.child_pid = None
		self.command = None
		self.url = None
		
	def fork_player(self, command, url):
		self.command = command
		self.url = url
		
		args = "%s %s" % (self.command, self.url)
		command_split = args.split(" ")
		
		print args
		
		pid = os.fork();
		if pid == -1: #fork error
			perror("fork")
			self.child_pid = None
		elif pid == 0: #execute in child
			os.execvp(command_split[0], command_split)
		else: #child's pid, main process execution
			self.child_pid = pid
			
	def kill(self):
		if self.is_running() == True:
			try:
				os.kill(self.child_pid, signal.SIGINT)
				killedpid, stat = os.wait()
			except OSError:
				sys.stderr.write("Process %s does not exist\n" % self.child_pid)
	
	def is_running(self):
		if self.child_pid != None:
			try:
				os.kill(self.child_pid, 0)
				return True
			except OSError:
				return False
		else:
			return False




# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import socket
import string
import time

class SOPStats:
	def __init__(self, host=None, port=None):
		self.sock = None
		self.host = host
		self.port = port
		
	def set_host(self, host):
		self.host = host
		
	def set_port(self, port):
		self.port = port
		
	def connect_to_server(self):
		try:
			self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			self.sock.connect((self.host, self.port))
			return True
		except:
			return False
		
	def send_msg(self, msg):
		sent = self.sock.send(msg)
		if sent == 0:
			raise RuntimeError, "socket connection broken"
			
	def receive_msg(self):
		BUFSIZE = 128
		msg = ''
		
		while string.count(msg, "\n") == 0:
			chunk = self.sock.recv(BUFSIZE)
				
			if chunk == '':
				raise RuntimeError, "socket connection broken"
			msg = msg + chunk
			
		return msg
		
	def update_stats(self):
		try:
			if self.connect_to_server() == True:
				self.send_msg("state\ns\n")
				self.send_msg("state\ns\n")
				stats = self.receive_msg()
	
				while len(stats.split(" ")) > 7:
					stats = stats.replace("  ", " ")
		
				stats = stats.split(" ")
				self.sock.close()
		
				return stats
			else:
				return None
		except:
			return None




import os, shutil

i = 1

for f in os.listdir('/home/jason/Downloads/sopcast-translations'):
	if len(f) > 5:
		shutil.copyfile('/home/jason/Downloads/sopcast-translations/%s' % (f), '/home/jason/Downloads/sopcast-translations/fixed/%s' % (f[15:]))

for f in os.listdir('/home/jason/Downloads/sopcast-translations/fixed'):
	print "%d: %s" % (i, f)
	i = i + 1
	
	for line in open('/home/jason/Downloads/sopcast-translations/fixed/%s' % (f)):
		if "Last-Translator" in line:
			print line



# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import DatabaseOperations
import os
import sys
from xml.dom.minidom import parse

class Language:
	ENGLISH = 0
	CHINESE = 1


class ImportChannelGuide(object):
	def __init__(self):
		self.db_operations = DatabaseOperations.DatabaseOperations()
	
	def getText(self, nodelist):
		rc = ""
	 	for node in nodelist:
			if node.nodeType == node.TEXT_NODE:
				rc = node.data
		return rc
		
	def attribute_exists(self, element, attribute):
		found = False
		
		if element.getAttribute(attribute) != "":
			found = True
		
		return found
		
	def get_english(self, element):
		value = None
		if self.attribute_exists(element, "en"):
			if element.attributes["en"].value != "":
				value = element.attributes["en"].value
				
		if value == None:
			if self.getText(element.childNodes) != "":
				value = self.getText(element.childNodes)
		
		if value == None:
			if self.attribute_exists(element, "cn"):
				if element.attributes["cn"].value != "":
					value = element.attributes["cn"].value
		
		return value
		
		
	def get_chinese(self, element):
		value = None
		if self.attribute_exists(element, "cn"):
			if element.attributes["cn"].value != "":
				value = element.attributes["cn"].value
				
		if value == None:
			if self.getText(element.childNodes) != "":
				value = self.getText(element.childNodes)
		
		if value == None:
			if self.attribute_exists(element, "en"):
				if element.attributes["en"].value != "":
					value = element.attributes["en"].value
		
		return value
		
	def get_default(self, element):
		value = None
		
		if self.getText(element.childNodes) != "":
			value = self.getText(element.childNodes)
		
		if value == None:
			if self.attribute_exists(element, "en"):
				if element.attributes["en"].value != "":
					value = element.attributes["en"].value
		
		if value == None:
			if self.attribute_exists(element, "cn"):
				if element.attributes["cn"].value != "":
					value = element.attributes["cn"].value
		
		return value
	
	def get_child_tag_translation(self, element, tag_name, language=None):
		value = None
		
		temp = element.getElementsByTagName(tag_name)[0]
		
		if language == Language.ENGLISH:
			value = self.get_english(temp)
		elif language == Language.CHINESE:
			value = self.get_chinese(temp)
		elif language == None:
			value = self.get_default(temp)
			
		return value
		
	def get_tag_translation(self, element, language=None):
		value = None
		
		if language == Language.ENGLISH:
			value = self.get_english(element)
		elif language == Language.CHINESE:
			value = self.get_chinese(element)
		elif language == None:
			value = self.get_default(element)
			
		return value
	
	def get_tag_attribute(self, element, attribute):
		value = None
		if self.attribute_exists(element, attribute):
			if element.attributes[attribute].value != "":
				value = element.attributes[attribute].value
		
		return value
		
	def get_child_tag_attribute(self, element, child_tag, attribute):
		return self.get_tag_attribute(element.getElementsByTagName(child_tag)[0], attribute)
		
	def get_child_tag_value(self, element, tag_name):
		return self.get_tag_value(element.getElementsByTagName(tag_name)[0])
		
	def get_tag_value(self, element):
		value = None
		while value == None:
			if self.getText(element.childNodes) != "":
				value = self.getText(element.childNodes)
			else:
				if len(element.childNodes) > 0:
					for el in element.childNodes:
						if element.getElementsByTagName(el.tagName)[0] != "":
							temp = element.getElementsByTagName(el.tagName)[0]
							self.get_tag_value(temp)
						element = el
				else:
					value = ""
		return value
		
	def format_description(self, description):
		value = None
		
		if description[:len("Description:")].lower() == "Description:".lower():
			value = description[len("Description:"):len(description)]
		else:
			value = description
			
		return value.lstrip()
		
	def get_channel_group_info(self, channel_group):
		info = [int(self.get_tag_attribute(channel_group, "id")),
			int(self.get_tag_attribute(channel_group, "type")),
			self.get_tag_translation(channel_group, Language.ENGLISH),
			self.get_tag_translation(channel_group, Language.CHINESE),
			self.get_tag_value(channel_group),
			self.format_description(self.get_tag_attribute(channel_group, "description"))]
		
		return info
		
	def get_channel_info(self, channel, channel_group_id):
		info = [int(self.get_tag_attribute(channel, "type")),
			int(self.get_tag_attribute(channel, "btype")),
			self.get_tag_attribute(channel, "language"),
			self.get_child_tag_translation(channel, "name", Language.ENGLISH),
			self.get_child_tag_translation(channel, "name", Language.CHINESE),
			self.get_child_tag_translation(channel, "name"),
			int(self.get_child_tag_value(channel, "status")),
			self.get_child_tag_translation(channel, "region", Language.ENGLISH),
			self.get_child_tag_translation(channel, "region", Language.CHINESE),
			self.get_child_tag_translation(channel, "region"),
			self.get_child_tag_translation(channel, "class", Language.ENGLISH),
			self.get_child_tag_translation(channel, "class", Language.CHINESE),
			int(self.get_child_tag_translation(channel, "class")),
			int(self.get_child_tag_value(channel, "user_count")),
			int(self.get_child_tag_value(channel, "sn")),
			int(self.get_child_tag_value(channel, "visit_count")),
			self.get_child_tag_value(channel, "start_from"),
			self.get_child_tag_value(channel, "stream_type"),
			int(self.get_child_tag_value(channel, "kbps")),
			int(self.get_child_tag_value(channel, "qs")),
			int(self.get_child_tag_value(channel, "qc")),
			self.get_child_tag_value(channel, "sop_address"),
			self.format_description(self.get_child_tag_translation(channel, "description", Language.CHINESE)),
			self.format_description(self.get_child_tag_value(channel, "description")),
			channel_group_id]
			
		return info

	def update_database(self, url):
		self.db_operations.truncate_channel_groups()
		self.db_operations.truncate_channels()
		
		f = open(url)
		doc = parse(f)
		f.close()

		for channel_group in doc.getElementsByTagName("group"):			
			channels = channel_group.getElementsByTagName("channel")			
			
			if len(channels) > 1:
				channel_group_info =  self.get_channel_group_info(channel_group)
				self.db_operations.insert_channel_group(channel_group_info)	
				for channel in channels:
					self.db_operations.insert_channel(self.get_channel_info(channel, channel_group_info[0]))
				channels = None
		channel_group = None
		
		self.db_operations.insert_channel_group([sys.maxint, 0, 'Other', 'Other', 'Other', 'Channel group to catch all un-categorized channels'])
		
		for channel_group in doc.getElementsByTagName("group"):	
			channels = channel_group.getElementsByTagName("channel")
			if len(channels) == 1:
				for channel in channels:
					self.db_operations.insert_channel(self.get_channel_info(channel, sys.maxint))
					
		self.db_operations.commit_channel_guide()
		doc = None
		
if __name__ == '__main__':
	i = ImportChannelGuide()
	i.update_database()
	
	while 1: i =1



#! /usr/bin/python
# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""VLC Widget classes.

This module provides two helper classes, to ease the embedding of a
VLC component inside a pygtk application.

VLCWidget is a simple VLC widget.

DecoratedVLCWidget provides simple player controls.

$Id$
"""

import gtk
import sys
import vlc_1_0_x
import vlc_1_1_x

from WindowingTransformations import WindowingTransformations

from gettext import gettext as _

class VLCWidget(gtk.DrawingArea):
	"""Simple VLC widget.

	Its player can be controlled through the 'player' attribute, which
	is a vlc.MediaPlayer() instance.
	"""
	def __init__(self, container, parent):
		gtk.DrawingArea.__init__(self)
		self.parent_cls = parent
		self.container = container
		
		try:
			instance=vlc_1_0_x.Instance()
			self.player=instance.mediacontrol_new_from_instance()
			self.parent_cls.config_manager.uses_new_bindings(False)
		except(Exception):
			instance = vlc_1_1_x.Instance()
			self.player=instance.media_player_new()
			self.parent_cls.config_manager.uses_new_bindings(True)
				
		if self.parent_cls.config_manager.uses_new_bindings():
			def handle_embed(*args):
				if sys.platform == 'win32':
					self.player.set_hwnd(self.window.handle)
				else:
					self.player.set_xwindow(self.window.xid)
				return True
				
			self.add_events(gtk.gdk.BUTTON_PRESS_MASK)
			self.connect("button_press_event", self.on_mouse_button_clicked)				
				
			self.connect("map", handle_embed)
			
			self.wt = WindowingTransformations(self.container, self.parent_cls)
	
			self.container.get_toplevel().add_events(gtk.gdk.KEY_PRESS_MASK)
			self.container.get_toplevel().connect("key-press-event", self.on_key_press)
		else:
			self.wt = WindowingTransformations(self, self.parent_cls)
			self.media_playing = False
		
		self.set_size_request(320, 200)		
		self.modify_bg(gtk.STATE_NORMAL, self.get_colormap().alloc_color("black"))		
		self.container.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("black"))
		self.is_fs = False
		self.is_fw = False
		
		

	def on_key_press(self, widget, event, data=None):
		if event.keyval == gtk.keysyms.Escape:
			if self.is_fs:
				self.toggle_fullscreen()
		elif gtk.gdk.keyval_name(event.keyval) in ["f", "F"]:
			if self.is_playing():
				self.toggle_fullscreen()
		elif gtk.gdk.keyval_name(event.keyval) in ["h", "H"]:
			if not self.is_fs:
				self.toggle_fullwindow()
		return False
		
	def on_mouse_button_clicked(self, widget, event):
		if event.type == gtk.gdk._2BUTTON_PRESS:
			if self.is_playing():
				self.toggle_fullscreen()
		else:
			return True
			
	def toggle_fullscreen(self):
		if self.is_fs:
			self.unfullscreen()
		else:
			self.fullscreen()

	def toggle_fullwindow(self):
		if self.is_fw:
			self.unfullwindow()
		else:
			self.fullwindow()

	def set_media_url(self, url):
		self.player.set_mrl(url)
		
	def play_media(self):
		if self.parent_cls.config_manager.uses_new_bindings():
			self.player.play()
			if len(self.container.get_children()) == 0:
				self.container.add(self)
			
			self.show()
		else:
			self.realize()
			self.player.set_visual(self.window.xid)
			self.player.start(0)
			self.media_playing = True	

	def is_playing(self):
		if self.parent_cls.config_manager.uses_new_bindings():
			return self.player.is_playing()
		else:
			return self.media_playing
		
	def media_loaded(self):
		return self.is_playing()
		
	def resume_media(self):
		self.player.resume()
		
	def stop_media(self):
		self.player.stop()
		
	def pause_media(self):
		self.player.pause()
		
	def exit_media(self):
		exit = True
		
	def display_text(self, text):
		self.player.display_text("%s" % text, 0, 5000)
		
	def is_fullscreen(self):
		return self.is_fs
		
	def fullscreen(self):
		self.wt.fullscreen()
		
		if self.parent_cls.config_manager.uses_new_bindings():
			self.is_fs = True
	
	def unfullscreen(self):
		self.wt.unfullscreen()
		if self.parent_cls.config_manager.uses_new_bindings():
			self.is_fs = False
		
	def fullwindow(self):
		self.wt.fullwindow()
		
		if self.parent_cls.config_manager.uses_new_bindings():
			self.is_fw = True
	
	def unfullwindow(self):
		self.wt.unfullwindow()
		
		if self.parent_cls.config_manager.uses_new_bindings():
			self.is_fw = False
	
	def set_volume(self, level):
		if self.parent_cls.config_manager.uses_new_bindings():
			self.player.audio_set_volume(level)
		else:
			self.player.sound_set_volume(level)
		
	def screenshot(self):
		return self.player.snapshot(0)




# Copyright (C) 2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import fork
import sys
import threading
import time
import os
import gtk
import FileDownload
import ImportChannelGuide
import DatabaseOperations
import datetime
import pySopCastConfigurationManager
import locale
import gettext

cur_locale = locale.setlocale(locale.LC_ALL, "")

gtk.glade.bindtextdomain("sopcast-player")
gtk.glade.textdomain("sopcast-player")

gettext.bindtextdomain("sopcast-player")
gettext.textdomain("sopcast-player")

lang = gettext.translation("sopcast-player", languages=[cur_locale], fallback=True)
lang.install('sopcast-player')

class UpdateChannelGuideThread(threading.Thread):
	def __init__ (self, parent):
		threading.Thread.__init__(self)
		self.parent = parent
		self.downloader = None
		self.guide_import = None
		self.db_operations = None
		self.daemon = True
		self.running = True
		self.updated = False
		self.last_updated = None
		
	def run(self):
		self.running = True
		chinese = False
		handler_blocked = False		
		
		gtk.gdk.threads_enter()
		self.parent.update_channel_guide_progress.set_text(_("Contacting Server"))
		gtk.gdk.threads_leave()
		
		gtk.gdk.threads_enter()
		self.parent.refresh_channel_guide.set_sensitive(False)
		gtk.gdk.threads_leave()
		
		try:
			self.downloader = FileDownload.FileDownload()
			self.downloader.download_file(self.parent.channel_guide_url, os.path.expanduser('~/.pySopCast/channel_guide.xml'), self.report_progress)
			self.downloader = None
	
			gtk.gdk.threads_enter()
			self.parent.update_channel_guide_progress.set_text(_("Updating Database"))
			gtk.gdk.threads_leave()
		
			self.guide_import = ImportChannelGuide.ImportChannelGuide()
			self.guide_import.update_database(os.path.expanduser('~/.pySopCast/channel_guide.xml'))
			self.guide_import = None
	
			gtk.gdk.threads_enter()
			self.parent.treeview_selection.handler_block(self.parent.treeview_selection_changed_handler)
			gtk.gdk.threads_leave()
	
			handler_blocked = True
	
			gtk.gdk.threads_enter()
			self.parent.channel_treeview.set_model()
			gtk.gdk.threads_leave()
	
			gtk.gdk.threads_enter()
			self.parent.channel_treeview.get_selection().unselect_all()
			gtk.gdk.threads_leave()
	
			gtk.gdk.threads_enter()
			self.parent.channel_treeview_model.clear()
			gtk.gdk.threads_leave()
		
			self.db_operations = DatabaseOperations.DatabaseOperations()	
			if self.parent.channel_guide_language == _("English"):
				channel_groups = self.db_operations.retrieve_channel_groups()
			else:
				channel_groups = self.db_operations.retrieve_channel_groups_cn()

			for channel_group in channel_groups:
				gtk.gdk.threads_enter()
				channel_group_iter = self.parent.channel_treeview_model.append(None, self.parent.prepare_row_for_channel_treeview_model(channel_group))
				gtk.gdk.threads_leave()
		
				if self.parent.channel_guide_language == _("English"):
					channels = self.db_operations.retrieve_channels_by_channel_group_id(channel_group[0])
				else:
					channels = self.db_operations.retrieve_channels_by_channel_group_id_cn(channel_group[0])

				for channel in channels:
					gtk.gdk.threads_enter()
					self.parent.channel_treeview_model.append(channel_group_iter, self.parent.prepare_row_for_channel_treeview_model(channel))
					gtk.gdk.threads_leave()
		
			self.db_operations = None
	
			gtk.gdk.threads_enter()
			self.parent.channel_treeview.set_model(self.parent.channel_treeview_model)
			gtk.gdk.threads_leave()
	
			gtk.gdk.threads_enter()
			self.parent.update_channel_guide_progress.set_text(_("Completed"))
			gtk.gdk.threads_leave()
	
			t = datetime.datetime.now()

			config_manager = pySopCastConfigurationManager.pySopCastConfigurationManager()
			config_manager.set("ChannelGuide", "last_updated", time.mktime(t.timetuple()))
			config_manager.write()
	
			self.updated = True
		except(Exception):
			gtk.gdk.threads_enter()
			if self.parent.update_channel_guide_progress != None:
				self.parent.update_channel_guide_progress.set_text(_("Server Down"))
			gtk.gdk.threads_leave()
		finally:
			time.sleep(1.5)
			gtk.gdk.threads_enter()
			if self.parent.update_channel_guide_progress != None:
				self.parent.update_channel_guide_progress.hide()
				self.parent.channel_guide_label.show()
			gtk.gdk.threads_leave()
			
		gtk.gdk.threads_enter()
		self.parent.refresh_channel_guide.set_sensitive(True)
		gtk.gdk.threads_leave()
		
		if handler_blocked == True:
			gtk.gdk.threads_enter()
			self.parent.treeview_selection.handler_unblock(self.parent.treeview_selection_changed_handler)
			gtk.gdk.threads_leave()

		self.running = False
		
	def report_progress(self, numblocks, blocksize, filesize):
		try:
			percent = min((numblocks*blocksize*100)/filesize, 100)
		except:
			percent = 100
		
		gtk.gdk.threads_enter()
		self.parent.update_channel_guide_progress.set_text("%s: %d%%" % (_("Downloading"), percent))
		gtk.gdk.threads_leave()
		
		gtk.gdk.threads_enter()
		self.parent.update_channel_guide_progress.set_fraction(float(percent / 100.0))
		gtk.gdk.threads_leave()		

